/*
 * Bedroom Buddy
 */
//This is where you need to paste your configuration snippet: template id, template name and authentication token 
//that you got when creating your device on Blynk.
#define BLYNK_TEMPLATE_ID ""
#define BLYNK_TEMPLATE_NAME ""
#define BLYNK_AUTH_TOKEN ""   

#define DHT_PIN 15
#define DHT_TYPE DHT22
#define LED_PIN 13
#define LED_COUNT 12
#define LIGHT_SENSOR_PIN 32


#define BLYNK_PRINT Serial

#include <WiFi.h>
#include <WiFiClient.h>
#include <BlynkSimpleEsp32.h>
#include <DHT.h>
#include <Adafruit_NeoPixel.h>

// enter your WiFi credentials.
char ssid[] = "";
char pass[] = "";

DHT dht(DHT_PIN, DHT_TYPE);
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);
BlynkTimer timer;

float humidity;
int temperature;
uint32_t color;
int lightSensorVal;
bool changeSheets = false; //True on sheet changing day
bool ledsOn = true;

void setup() {
  Serial.begin(115200);
  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, pass);
  dht.begin();
  pinMode(LIGHT_SENSOR_PIN, INPUT);
  timer.setInterval(3600000L,checkHumidity); // checks the humidity level every hour.
  timer.setInterval(30000L,checkLight);  // checks if the lights have been turned off, every 30 seconds.
  timer.setInterval(2000L, flashStrip);
  strip.begin();         
  strip.show();
  strip.setBrightness(150);
}

void loop() {
  Blynk.run();
  timer.run();
}
// This function reads the humidity level from the Bedroom Buddy and notifies you if it's inder 50% (dry)
// You can adjust the limit to you liking.
void checkHumidity(){
   humidity = dht.readHumidity();
   Serial.print("Humidity % in bedroom: " );
   Serial.println(humidity);
   if(humidity < 50){
     Blynk.virtualWrite(V0, 1); //set slider to 1
     Blynk.virtualWrite(V0, 0);
   }
}

void checkLight(){
  lightSensorVal = analogRead(LIGHT_SENSOR_PIN);
  Serial.print("Light level: ");
  Serial.println(lightSensorVal);
  if (lightSensorVal >= 4000){ //bedroom is blacked out
    Blynk.virtualWrite(V0,2); //set slider to 2 
    Blynk.virtualWrite(V0,0);
  }
}

// will be called when Make update the temperature every 2 hours.
BLYNK_WRITE(V1) { 
  temperature = param.asInt(); //get the current temp fromt the widget
  switch(temperature){
    case 0 ... 15: // Temperature from 0 to 15 degrees Celsius. You can adjust these cases according to your preference
      color = strip.Color(0, 0, 255); // Blue
      break;
    case 16 ... 24: 
      color = strip.Color(255, 255, 0); // Yellow
      break;
    case 25 ... 30:
      color = strip.Color(255, 165, 0); // Orange
      break;
    default: // Temperature above 30 degrees Celsius
      color = strip.Color(255, 0, 0); // Red
      break;  
  }
  for (int i = 0; i < LED_COUNT; i++) {
   strip.setPixelColor(i, color);
   strip.show();
  }                
}
// Once a day the make scenario will check if today is the event day.
BLYNK_WRITE(V2) {
  if(param.asInt() == 1){ // if the event is on today's calendar, the switch will turn on
    changeSheets = true;
  } else {
    changeSheets = false;
  }
}
// If today is sheet changing day, the LEDs will flash during the day to remind me.
void flashStrip(){
  if(changeSheets){
    if(ledsOn) {
      turnOffLed();
      ledsOn = false;
    } else {
      turnOnLed();
      ledsOn = true;
    }
  }
} 

void turnOnLed() {
  for (int i = 0; i < LED_COUNT; i++) {
    strip.setPixelColor(i, color);
  }
  strip.show();
}

void turnOffLed() {
  strip.fill(0); // Turn off all LEDs
  strip.show();
} 
